package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.DataLengthException;
import cn.com.infosec.crypto.StreamCipher;
import cn.com.infosec.crypto.params.KeyParameter;

public class ISAACEngine implements StreamCipher {
   private final int sizeL = 8;
   private final int stateArraySize = 256;
   private int[] engineState = null;
   private int[] results = null;
   private int a = 0;
   private int b = 0;
   private int c = 0;
   private int index = 0;
   private byte[] keyStream = new byte[1024];
   private byte[] workingKey = null;
   private boolean initialised = false;

   private void setKey(byte[] keyBytes) {
      this.workingKey = keyBytes;
      if (this.engineState == null) {
         this.engineState = new int[256];
      }

      if (this.results == null) {
         this.results = new int[256];
      }

      int var2;
      for(var2 = 0; var2 < 256; ++var2) {
         this.results[var2] = 0;
         this.engineState[var2] = 0;
      }

      this.c = 0;
      this.b = 0;
      this.a = 0;
      this.index = 0;
      int var3;
      byte[] var4 = new byte[var3 = keyBytes.length + (keyBytes.length & 3)];
      int keyBytes1 = keyBytes.length;
      System.arraycopy(keyBytes, 0, var4, 0, keyBytes1);

      for(keyBytes1 = 0; keyBytes1 < var3; keyBytes1 += 4) {
         var2 = keyBytes1 >> 2;
         this.results[var2] = this.byteToIntLittle(var4, keyBytes1);
      }

      int[] var8 = new int[8];

      for(keyBytes1 = 0; keyBytes1 < 8; ++keyBytes1) {
         var8[keyBytes1] = -1640531527;
      }

      for(keyBytes1 = 0; keyBytes1 < 4; ++keyBytes1) {
         this.mix(var8);
      }

      byte keyBytes2 = 0;

      for(var3 = keyBytes2; var3 < 2; var3 = keyBytes1) {
         keyBytes2 = 0;

         for(int var10 = keyBytes2; var10 < 256; var10 = keyBytes1) {
            int var5;
            for(keyBytes1 = 0; keyBytes1 < 8; ++keyBytes1) {
               var5 = var8[keyBytes1];
               int var6;
               if (var3 < 1) {
                  var6 = this.results[var10 + keyBytes1];
               } else {
                  var6 = this.engineState[var10 + keyBytes1];
               }

               var8[keyBytes1] = var5 + var6;
            }

            this.mix(var8);

            for(keyBytes1 = 0; keyBytes1 < 8; ++keyBytes1) {
               var5 = var10 + keyBytes1;
               this.engineState[var5] = var8[keyBytes1];
            }

            keyBytes1 = var10 + 8;
         }

         keyBytes1 = var3 + 1;
      }

      this.isaac();
      this.initialised = true;
   }

   private void isaac() {
      int var10001 = this.b;
      int var1;
      this.c = var1 = this.c + 1;
      this.b = var10001 + var1;

      for(var1 = 0; var1 < 256; ++var1) {
         int[] var2;
         int var3 = (var2 = this.engineState)[var1];
         switch (var1 & 3) {
            case 0:
               var10001 = this.a;
               this.a = var10001 ^ var10001 << 13;
               break;
            case 1:
               var10001 = this.a;
               this.a = var10001 ^ var10001 >>> 6;
               break;
            case 2:
               var10001 = this.a;
               this.a = var10001 ^ var10001 << 2;
               break;
            case 3:
               var10001 = this.a;
               this.a = var10001 ^ var10001 >>> 16;
         }

         int var4;
         this.a = var4 = this.a + var2[var1 + 128 & 255];
         var2[var1] = var4 = var2[var3 >>> 2 & 255] + var4 + this.b;
         int[] var10000 = this.results;
         int var5;
         this.b = var5 = var2[var4 >>> 10 & 255] + var3;
         var10000[var1] = var5;
      }

   }

   private void mix(int[] x) {
      x[0] ^= x[1] << 11;
      x[3] += x[0];
      x[1] += x[2];
      x[1] ^= x[2] >>> 2;
      x[4] += x[1];
      x[2] += x[3];
      x[2] ^= x[3] << 8;
      x[5] += x[2];
      x[3] += x[4];
      x[3] ^= x[4] >>> 16;
      x[6] += x[3];
      x[4] += x[5];
      x[4] ^= x[5] << 10;
      x[7] += x[4];
      x[5] += x[6];
      x[5] ^= x[6] >>> 4;
      x[0] += x[5];
      x[6] += x[7];
      x[6] ^= x[7] << 8;
      x[1] += x[6];
      x[7] += x[0];
      x[7] ^= x[0] >>> 9;
      x[2] += x[7];
      x[0] += x[1];
   }

   private int byteToIntLittle(byte[] x, int offset) {
      int var3 = offset + 1;
      return x[offset] & 255 | (x[var3++] & 255) << 8 | (x[var3++] & 255) << 16 | x[var3] << 24;
   }

   private byte[] intToByteLittle(int x) {
      byte[] var2;
      byte[] var10000 = var2 = new byte[4];
      var2[3] = (byte)x;
      var2[2] = (byte)(x >>> 8);
      var2[1] = (byte)(x >>> 16);
      var10000[0] = (byte)(x >>> 24);
      return var10000;
   }

   private byte[] intToByteLittle(int[] x) {
      byte[] var2 = new byte[x.length * 4];
      int var3 = 0;

      for(int var4 = 0; var3 < x.length; var4 += 4) {
         System.arraycopy(this.intToByteLittle(x[var3]), 0, var2, var4, 4);
         ++var3;
      }

      return var2;
   }

   public void init(boolean forEncryption, CipherParameters params) {
      if (params instanceof KeyParameter) {
         this.setKey(((KeyParameter)params).getKey());
      } else {
         throw new IllegalArgumentException("invalid parameter passed to ISAAC init - " + params.getClass().getName());
      }
   }

   public byte returnByte(byte in) {
      if (this.index == 0) {
         this.isaac();
         this.keyStream = this.intToByteLittle(this.results);
      }

      int var2;
      byte var10000 = (byte)(this.keyStream[var2 = this.index] ^ in);
      this.index = var2 + 1 & 1023;
      return var10000;
   }

   public void processBytes(byte[] in, int inOff, int len, byte[] out, int outOff) {
      if (this.initialised) {
         if (inOff + len <= in.length) {
            if (outOff + len <= out.length) {
               for(int var6 = 0; var6 < len; ++var6) {
                  if (this.index == 0) {
                     this.isaac();
                     this.keyStream = this.intToByteLittle(this.results);
                  }

                  int var7 = var6 + outOff;
                  byte[] var8 = this.keyStream;
                  int var9;
                  int var10001 = var9 = this.index;
                  out[var7] = (byte)(var8[var9] ^ in[var6 + inOff]);
                  this.index = var10001 + 1 & 1023;
               }

            } else {
               throw new DataLengthException("output buffer too short");
            }
         } else {
            throw new DataLengthException("input buffer too short");
         }
      } else {
         throw new IllegalStateException(this.getAlgorithmName() + " not initialised");
      }
   }

   public String getAlgorithmName() {
      return "ISAAC";
   }

   public void reset() {
      this.setKey(this.workingKey);
   }
}
